<?php

/**
 * 20/10/2015
 * @author Victor Novy
 */

namespace Teknisa\PED\Controllers;

use \Zeedhi\Framework\DTO\Request\Row;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Teknisa\PED\Services\ValidacoesEstoqueService;

class ValidacoesEstoqueController {

    protected $validacoesEstoqueService;
	protected $environment;


    public function __construct (ValidacoesEstoqueService $validacoesEstoqueService, $environment, $entityManager) {
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    /**
     * Busca os dodos relacionados a 'data de processamento' e realiza a validacao da 'data do lancemento'.
     *
     * @param Row $request
     * @param Response $response
     */
    public function validaDataLancamento(Row $request, Response $response) {
        try {
            $dadosValidacao = $request->getRow();

            $filial = $dadosValidacao['CDFILIAL'];
            $dtLancamento = $dadosValidacao['DTLANCTOEST'];
            $operacao = $dadosValidacao['OPERACAO'];
            $sistema = $dadosValidacao['IDSISTEMA'];
            $dtEmissao = $dadosValidacao['DTEMISSAO'];
            
            $this->validacoesEstoqueService->validaDataLancamento(
                $filial, $dtLancamento, $operacao, $sistema, $dtEmissao
            );
        } catch (\Exception $e) {
            $response->setError(new Error($e->getmessage(), $e->getCode()));
        }
    }


    /**
     * Procedimento que recebe uma row e pede pela
     * requisição da atualização de estoque para o
     * lançamento de estoque e unidade informados,
     * respeitando quaisquer flags recebidas.
     *
     * Envia no objeto response um DataSet com os códigos
     * dos produtos que apresentam irregularidades de
     * estoque, nos casos em que essa possibilidade se
     * aplica.
     *
     * Envia no objeto response um erro, quando aplicável
     *
     * @param Row      $request
     * @param Response $response
     */
    public function atualizaEstoque(Row $request, Response $response) {
        try {
            $payload = $request->getRow();

            #gets relevant data from payload
            $cdfilial              = $payload['CDFILIAL'];
            $nrlancestq            = $payload['NRLANCESTQ'];
            $verifyIrregularities  = $payload['IDVERIRREG'];
            $abortAtIrregularities = $payload['IDABRIRREG'];
            $seqItemCodes          = $payload['SEQITEMS'];

            #calls procedure validation, preparation and execution, receiving any irregular products, if applicable
            $irregularProducts = $this->validacoesEstoqueService->atualizaEstoque(
                $cdfilial, $nrlancestq, $verifyIrregularities, $abortAtIrregularities, $seqItemCodes
            );

            #sends data to frontend through response's dataset
            $response->addDataset(new DataSet("irregularProducts", $irregularProducts));
        } catch(\Doctrine\DBAL\DBALException $e) {
            #catches doctrine exceptions and prepares message for response's error
            $message = substr($e->getMessage(), strpos($e->getMessage(), 'ORA') + 10);
            $message = substr($message, 0, strpos($message, 'ORA') - 1);

            $response->setError(new Error($message, $e->getCode()));
        } catch (\Exception $e) {
            #catches generic error and sends message to frontend through response's error
            $response->setError(new Error($e->getmessage(), $e->getCode()));
        }
    }

    public function zeraInventario(Row $request, Response $response) {
        try {
            $dadosAtualizacao = $request->getRow();

            $filial = $dadosAtualizacao['CDFILIAL'];
            $lancamento = $dadosAtualizacao['NRLANCAMENTO'];
            $idordem = $dadosAtualizacao['IDORDEMCLC'];

            $this->validacoesEstoqueService->zeraInventario($filial, $lancamento, $idordem);
        } catch(\Doctrine\DBAL\DBALException $e) {
            $message = substr($e->getMessage(), strpos($e->getMessage(), 'ORA') + 10);
            $message = substr($message, 0, strpos($message, 'ORA') - 1);
            $response->setError(new Error($message, $e->getCode()));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getmessage(), $e->getCode()));
        }
    }

    public function consultaCustoQuantidadeProduto(Row $request, Response $response) {
        try {
            $dadosConsultaCusto = $request->getRow();

            $filial = $dadosConsultaCusto['CDFILIAL'];
            $tipoCusto = $dadosConsultaCusto['CDTIPOCUST'];
            $produto = $dadosConsultaCusto['CDPRODUTO'];
            $almoxarifado = $dadosConsultaCusto['CDALMOXARIFE'];
            $localEstoque = $dadosConsultaCusto['CDLOCALESTOQ'];
            $lote = $dadosConsultaCusto['NRLOTE'];
            $subLote = $dadosConsultaCusto['NRSUBLOTE'];
            $dtLancamento = $dadosConsultaCusto['DTLANCAMENTO'];

            $custoQuantidade = $this->validacoesEstoqueService->consultaCustoQuantidadeProduto(
                $filial, $produto, $dtLancamento, $tipoCusto, $almoxarifado, $localEstoque, $lote, $subLote
            );
            $response->addDataset(new DataSet('consultaCustoQuantidadeProduto', $custoQuantidade));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getmessage(), $e->getCode()));
        }
    }
    
    public function alterarEstrutura(Row $request, Response $response) {
        try {
            $operador = $request->getRow();
            $this->alterarEstruturaService($operador['CDFILIAL'], $this->environment->getCdoperador(), $operador['NRORG']);
            $row = array();
            $row['CDFILIAL'] = $operador['CDFILIAL'];
            $row['CDOPERADOR'] = $this->environment->getCdoperador();
            $row['NRORG'] = $operador['NRORG'];
            $dataSet = new DataSet('operador', array($row));
            $response->setDataSets(array($dataSet));
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    public function alterarEstruturaService($cdfilial, $cdoperador, $nrorg) {
        $operador = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $cdoperador,
                    'nrorg' => $nrorg
                )
            );
        $operador->setNrorgtrab($nrorg);
        $operador->setCdfilialselecionada($cdfilial);
        $operador->setNrorgselecionada($nrorg);
        $this->entityManager->persist($operador);
        $this->entityManager->flush();
        $this->environment->setCdFilial($cdfilial);
    }
    
}